#!/bin/env/python
import argparse
import gzip
import random
from collections import OrderedDict


def parse_cadd_file(file_object):
    header = file_object.readline().upper().strip('#').strip().split('\t')

    for line in file_object:
        entries = line.strip().split('\t')
        yield OrderedDict(zip(header, entries))


def get_site_identifiers(site):
    return site['CHROM'], site['POS'], site['REF'], site['ALT']


def get_consequences_per_site(file_object):

    file = parse_cadd_file(file_object)

    current_site = next(file)
    current_site_id = get_site_identifiers(current_site)
    consequences = [current_site]

    for next_site in file:

        next_site_id = get_site_identifiers(next_site)

        if next_site_id == current_site_id:
            consequences.append(next_site)
        else:
            yield consequences
            current_site = next_site
            current_site_id = next_site_id
            consequences = [current_site]

def get_header(file_path):
    with gzip.open(file_path) as f:
        for line in f:
            if '#' in line:
                return line

if __name__ == '__main__':
    parser = argparse.ArgumentParser('Script to generate a matched set of variants from another source based on CADD annotation.')
    parser.add_argument('--source', help='Source CADD annotations to draw matched variants from')
    parser.add_argument('--variants', help='CADD file with variants that you want to match. Must be sorted by site.')
    parser.add_argument('--output_variants', help='Output file for chosen variants.')
    parser.add_argument('--output_source', help='Output file for matched variants from source.')
    parser.add_argument('--min_af', default=0.05, type=int, help='Min allele frequency to consider for matched site')

    args = parser.parse_args()

    variants_header = get_header(args.variants)
    source_header = get_header(args.source)

    with gzip.open(args.source) as source, gzip.open(args.variants) as variants, gzip.open(args.output_variants, 'w') as output_variants, gzip.open(args.output_source, 'w') as output_source:

        output_variants.write(variants_header)
        output_source.write(source_header)

        variant_site_consequences = get_consequences_per_site(variants)
        source_site_consequences = get_consequences_per_site(source)


        for site_consequence in variant_site_consequences:
            site_categories = [x['CONSEQUENCE'] for x in site_consequence]

            if 'NON_SYNONYMOUS' in site_categories or 'STOP_GAINED' in site_categories:
                continue

            site = max(site_consequence, key=lambda x: int(x['CONSSCORE']))

            match = None
            for source_variant_consequence in source_site_consequences:

                site_categories = [x['CONSEQUENCE'] for x in source_variant_consequence]

                if 'NON_SYNONYMOUS' in site_categories or 'STOP_GAINED' in site_categories:
                    continue

                for source_variant in source_variant_consequence:
                    if source_variant['CONSEQUENCE'] == site['CONSEQUENCE'] and float(source_variant['TG_AF']) > args.min_af:
                        match = source_variant
                        break

                if match:
                    break

            if match:
                output_source.write('\t'.join(match.values()) + '\n')
                output_variants.write('\t'.join(site.values()) + '\n')
            else:
                print 'NO MATCH!'








